function [a,yhat,uhat, upi, ufoi] = direct(Data,Pars,U, mu1, mu2)

% Direct estimation of epidemiological parameters
% INPUT: Data = two columns (x,y) with with x = time or age and y = antibody levels            
%        Pars = [xmin xmax nseg deg lambda], parameters for
%                computation of a B-splines basis of degree 'deg'
%                on a uniform grid with 'nseg' intervals between
%                'xmin' and 'xmax'with smoothness parameter lambda
%        U = points to predict
%        mu1 = antibody level mean of the susceptible population
%        mu2 = antibody level mean of the infected population
%
% OUTPUT: a = coefficients
%         yhat = estimated values given X
%         uhat = predicted values given U
%         upi = predicted prevalence given U
%         ufoi = predicted force of infection given u 
%
% Kaatje Bollaerts, June 2012


x0 = Pars(1); x1 = Pars(2); ndx = Pars(3); deg = Pars(4); lambda = Pars(5)

%%ESTIMATION yhat

 %--construct B-splines basis for estimation
 B = bsplbase(Data(:,1), Pars);
 n = size(B, 2);

 %--construct penalty matrices
 E = eye(n);
 S = diff(E, 2);
 M = diff(E, 1);
 kappa = 1e6;

 %--L2 P-splines isotonic regression
 v = zeros(n - 1,1);
 repeat = 1;
 while repeat;
    V = diag(v);   
    a = (B' * B + lambda * S' * S +  kappa * M' * V * M) \ (B' * Data(:,2));
    v0 = v; 
    v = diff(a,1) < 0.0;
    repeat = sum(abs(v-v0)) > 0;
 end


%%PREDICTION uhat
 %--construct B-splines basis for prediction
 Bpred = bsplbase(U,Pars);

 %--prediction
 yhat = B * a;
 uhat = Bpred * a;


%%PREDICTION upi
 %--direct estimation prevalence 
 upi = (uhat - mu1)./ (mu2 - mu1);

%%PREDICTION ufoi
 %--compute first order derivative
 dx = (x1 - x0) / (ndx);
 t = x0 + dx * ((-deg + 1):(ndx-1));
 T = ones(size(U')) * t;
 UU = U' * ones(size(t));
 D = (UU - T) / dx;
 Bder = diff([zeros(size(U')), D <= 1]')';
 r = [2:length(t) 1];
 q = deg-1;
 for j = 1:q;
   Bder = (D .* Bder + (j + 1 - D) .* Bder(:, r)) / j;
 end;

 dx = (Pars(2)-Pars(1))/Pars(3);
 C = ((Pars(4)-j+1)*dx)^(-1)*(Pars(4)-j+1);
 ad = diff(a,1);
 d1 = C * Bder * ad;  
 
 %--direct estimation foi of infection  
 ufoi = d1 ./ (mu2 - uhat);








